//////////////////////////////////////////////////////////////////////////
//                                                                      //
// cosf.cpp -                                                           //
//                                                                      //
//                              //
//        .                          //
//                                                                      //
// Copyright (c) 2001-2008 RC Module                                    //
//                                                                      //
//  :  ..                                                 //
//                                                                      //
//   $Revision:: 1     $      $Date:: 11.05.11 16:33   $                //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#include "auxfd.h"

#ifdef __cplusplus
extern "C" {
#endif  // __cplusplus

typedef unsigned int uint;

#if !defined(__NM6403__) && !defined(__NM6405__)

uint mullUHi( uint a1, uint a2 )
{
unsigned long aa1 = a1;
unsigned long aa2 = a2;
return (aa1*aa2)>>32;

#else		//	    ..

uint mullUHi( uint , uint )
{
//	return a1*a2;
	__asm{
		extern Mul32HighU : label;
	ar5 = ar6-3;                                  //<31,12>
	delayed call Mul32HighU;
gr6 = [ar5];
gr7 = [--ar5];
	ar7 = ar6 with gr7 = gr6;
	pop ar6,gr6;
return;                                  //<31,2>
	}	//asm
	return 0;
#endif  // !defined(__NM6403__) && !defined(__NM6405__)
}

const unsigned long _pi = 0xC90FDAA22168C235l;
const unsigned long _pi2 = 0x6487ED5110B4611Al;

// cosf --
//
//    
//
//   -  
const uint c1 = 0x80000000; //	1/2!
const uint c2 = 0x0AAAAAAA; //	1/4!
const uint c3 = 0x005B05B0; //	1/6!
const uint c4 = 0x0001A01A; //	1/8!
const uint c5 = 0x0000049F; //	1/10!
const uint c6 = 0x00000009; //	1/12!

static float internal_cosf( uint uMnt, int Exp, int sgn ){

	uint quad = mullUHi( uMnt, uMnt );
	uint store = 0;
	//	   
	if ( Exp<-1 )
		quad >>= (-1-Exp)*2;
	else
		store = mullUHi( quad, c6 - store );	//	 <0.5    
	//	res = 1-quad*(c1-quad*(c2-quad*(c3-quad*(c4-quad*(c5-quad*c6)))))
	store = mullUHi( quad, c5 - store );
	store = mullUHi( quad, c4 - store );
	store = mullUHi( quad, c3 - store );
	store = mullUHi( quad, c2 - store );
	store = mullUHi( quad, c1 - store );
	if ( store ){
		store = -(int(store));	//	1-store;
		//	 , ,    
		store <<=1;
		store >>=9;
//		store = (store>>1) + (store&1);
		//if ( store==0x00800000 )
		//	Exp=0;	//	1
		//else
			Exp = -1;
	}
	else {
		Exp = 0; //	1
	}
    //   float
	return ConstructFloat( store, Exp, sgn, 0);
}

// sinf --
//
//    
//
//   -  
const uint s1 = 0x2AAAAAAA; //	1/3!
const uint s2 = 0x02222222; //	1/5!
const uint s3 = 0x000D00D0; //	1/7!
const uint s4 = 0x00002E3B; //	1/9!
const uint s5 = 0x0000006B; //	1/11!

static float internal_sinf( uint uMnt_orig, int Exp, int sgn ){

	//	   
	uint uMnt = uMnt_orig;
	if ( Exp<-1 )
		uMnt >>= (-1-Exp);
	uint quad = mullUHi( uMnt, uMnt );
	uint store = 0;
	if ( Exp==-1 )
		store = mullUHi( quad, s5 - store );	//	 <0.5    
	//	res = uMnt*( 1-quad*(c1-quad*(c2-quad*(c3-quad*(c4-quad*c5)))))
	store = mullUHi( quad, s4 - store );
	store = mullUHi( quad, s3 - store );
	store = mullUHi( quad, s2 - store );
	store = mullUHi( quad, s1 - store );
	store = mullUHi( uMnt_orig, -store );

//	Exp = -1;
	if ( store ){
		while((int(store))>0){
			store <<=1;
			Exp--;
		}
		store <<=1;
		store >>=9;
//		store = (store>>1) + (store&1);
	}
	else {
		Exp = -0x7F; //	0
	}
    //   float
	return ConstructFloat( store, Exp, sgn, 0);
}

#define isDenormal( x ) ((int(x>>32))>0)

float __sinCosf( float f, int isCos ){
	int Mnt;               //   
	int Exp, Sgn, Spc;     //  ,     - 
    //   float
	int tmpExp;	//	,  :(
	int tmpSgn;
	DestructFloat( f, Mnt, tmpExp, tmpSgn, Spc);
	Sgn = tmpSgn & (~isCos);
	//	1 = 0x00800000, 1<=Mnt<2
	Exp = tmpExp;
	uint uMnt = Mnt;
	uMnt <<=8;
	//  1 = 0x80000000
	//	Exp==0 => 1<=x<2
	//	Exp==1 => 2<=x<4
	//	Exp==2 => 4<=x<8
	//	Exp>2 => 8<=x

	//	sin(x+ 2 Pi n) = sin(x)
	if ( Exp >2 ){
		if ( Exp > 20 ){								//   ,  
			if ( Exp ==128 && Mnt!=0 )					// NaN
				return f;
			if (!_FPIEEE_Flags)
				_FPIEEE_Signal( _FPIEEE_TLoss );
			return ConstructFloat( 0, 0, 0, 0);
		}
		unsigned long ulArg = uMnt;
		ulArg <<= 32;
		ulArg = ulArg % (_pi2>>(Exp-3));
		ulArg <<=(Exp-3);
		Exp = 3;
		while( isDenormal(ulArg) ){
			ulArg <<=1;
			Exp--;
		}
		uMnt = ulArg>>32;
	}
	//  x<8
	//	sin( x-Pi ) = -sin(x)
	//	cos( x-Pi ) = -cos(x)
	while ( Exp ==2 ){
		unsigned long ulArg = uMnt;
		ulArg <<= 32;
		ulArg = ulArg - _pi2;
		while( isDenormal(ulArg) ){
			ulArg <<=1;
			Exp--;
		}
		uMnt = ulArg>>32;
		Sgn = 1-Sgn;
	}
	//	sin( x-Pi/2 ) = -cos(x)
	//	cos( x-Pi/2 ) = sin(x)
	while ( Exp ==1 ){
		unsigned long ulArg = uMnt;
		ulArg <<= 32;
		ulArg = ulArg - _pi2;
		while( isDenormal(ulArg) ){
			ulArg <<=1;
			Exp--;
		}
		uMnt = ulArg>>32;
		Sgn ^= isCos;
		isCos = 1-isCos;
	}
	//	 
	if ( Exp ==0 ){
		unsigned long ulArg = uMnt;
		ulArg <<= 32;
		if ( ulArg > _pi ){
			ulArg = ulArg - _pi;
			Sgn ^= isCos;
		}
		else
			ulArg = _pi - ulArg;
		while( isDenormal(ulArg) ){
			ulArg <<=1;
			Exp--;
		}
		uMnt = ulArg>>32;
		isCos = 1-isCos;
	}
	if ( uMnt==0x80000000 )	//	     0x80000000*0x80000000
		uMnt++;
	if ( isCos )
		return internal_cosf( uMnt, Exp, Sgn );
	else
		return internal_sinf( uMnt, Exp, Sgn );
}

#ifdef __cplusplus
}
#endif  // __cplusplus
