//////////////////////////////////////////////////////////////////////////
//                                                                      //
// sqrtf.cpp -                                                          //
//                                                                      //
//                              //
//        .                          //
//                                                                      //
// Copyright (c) 2001 RC Module Inc.                                    //
//                                                                      //
//  :  ..                                                 //
//                                                                      //
//   $Revision:: 1     $      $Date:: 2/14/06 8:09p    $                //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#include "math.h"
#include "auxfd.h"

/*
            sqrt, sqrtf, sqrtl
            double sqrt(double x);
            float sqrt(float x); [C++ only]
            long double sqrt(long double x); [C++ only]
            float sqrtf(float x); [optional]
            long double sqrtl(long double x); [optional] 

            The function returns the square root of x, x^(1/2).
*/

#ifdef __cplusplus
extern "C" {
#endif

// sqrtf --
//
//      
//
//    --.
//   :   Xi  (Xi)^2 ,  (Xi)^2   
//   .   Xi -   
float sqrtf(float f){
	int m;                             //   
    int Xi=0;                          //  Xi
    int Xi2=0;                         //  (Xi)^2
    int p2i2 = 0x20000000;             //  (2^i)^2
    int p2i = 0x40000000;              //  2^i
	int e, s, sc;                      //     
    int i=0;                           //   

    //   
    DestructFloat(f, m, e, s, sc);

    if ( s )
    	return ConstructFloat(1, 0x80, 0, 0); // x<0 -> NAN

    if ( e == 0x80 ){ // X = inf, NAN
        return f;
    }

    //      
    m <<= 7;

    //         ,  , 
    //           

    if ( e & 1 ){       //   
        e--;            //  , = ..,. * 2
    }
    else{               //   
        m>>=1;          //  , = .,..
    }

    e>>=1;              //  e = e/2

    do {
        if (m>Xi2){
            Xi2 += (Xi>>(i++)) + p2i2;    //  X[i+1]^2 = X[i]^2 + 2*X[i]*(2>>i) + (2>>i)^2
            Xi += p2i;
        }
        else{
            Xi2 += p2i2 - (Xi>>(i++));
            Xi -= p2i;
        }
        p2i>>=1;
    }while (p2i2>>=2);

    //  p2i2      
    do {
        if (m>Xi2){
            Xi2 += Xi>>(i++);
            Xi += p2i;
        }
        else{
            Xi2 -= Xi>>(i++);
            Xi -= p2i;
        }
    }while (p2i>>=1);

    //     
    Xi>>=6;
    //       -     
    Xi = (Xi >> 1) + (Xi & 1);
    //   
	return ConstructFloat(Xi, e, s, 0);
}

#ifdef __cplusplus
}
#endif
