//**************************************************************************//
//
//     RC MODULE, LTD.
//
//     NAME:             FHT (Fast Hadamard Transform)
//                       256 points (2**8)  
//
//     Revision Date:    17/03/99
//     Author:           Vitali Kashkarov
//
//     DATA
//           Input data:  less or equal 5 bits
//           Result data: no more than 13 bits
//     USAGE
//           This is main program that initialize an input array and
//           calls FHT assembly routines( Hand optimized )
//**************************************************************************//
#include "Hadam256.h"
#include <time.h>

int main()
{
    clock_t t0, t1;
    int i;
    unsigned long ChkSum = 0;

    // Initialization of Input buffer;
    for ( i = 0; i < 32; i++ )
    {
        GBuff[i]   = 0x03020100fffefdfc;
    }

    // Start Timer
    t0 = clock();
        // Call the routine to calculate first 3 steps of FHT
        Steps_1_3( GBuff, LBuff );
        // Call the routine to calculate next 5 steps of FHT
        Steps_4_8( LBuff, GBuff );
    // Timer Stop;
    t1 = clock();

    // Calculate checksum
    for ( i = 0; i < 32; i++ )
    {
        ChkSum ^= GBuff[i];
    }

    // Compare ChkSum to expected value.
    if (ChkSum != 0x0000ff00ff800180)
        // Return error.
        return -1;
    else
        // Return number of cycles spent on calculation of 256-point FHT
        return t1 - t0;
}

//**************************************************************************//
//                              End of file
//**************************************************************************//
